/*
 * Main class for playing in the Wumpus world
 */

package edu.umn.d.cs5541.program3.wumpusworld;

import java.io.BufferedReader;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashSet;

/**
 *
 * Class used to run a WumpusWorld game
 */
public class Main {

    // Variable representing the Wumpus world
    private static int size;
    private static WumpusMaze maze;

    // Aspects of the current world
    private static int currentX;
    private static int currentY;
    private static int facingAngle;
    private static boolean hasArrow;

    // Values of visited/percepts
    private static boolean[][] visited;
    private static boolean[][] glitter;
    private static boolean[][] smell;
    private static boolean[][] breeze;

    // Initialize the world and the percept information
    private static void initialize(int theSize) {
        size = theSize;
        maze = new WumpusMaze(size);
        visited = new boolean[size][size];
        glitter = new boolean[size][size];
        smell = new boolean[size][size];
        breeze = new boolean[size][size];
        for (int x = 0; x < size; x++) {
            for (int y = 0; y < size; y++) {
                visited[x][y] = false;
                glitter[x][y] = false;
                smell[x][y] = false;
                breeze[x][y] = false;
            }
        }
    }

    // Show a representation of the maze percepts
    private static void showBoard() {
        for (int y = size - 1; y >= 0; y--) {
            for (int x = 0; x < size; x++) {
                System.out.print("|");
                if ((x == currentX) && (y == currentY)) {
                    switch (facingAngle) {
                        case 0: System.out.print(">"); break;
                        case 90: System.out.print("^"); break;
                        case 180: System.out.print("<"); break;
                        case 270: System.out.print("v"); break;
                        default: System.out.println("E"); break;
                    }
                }
                else
                    System.out.print(" ");
                System.out.print((visited[x][y] ? "." : " "));
                System.out.print((glitter[x][y] ? "g" : " "));
                System.out.print((smell[x][y] ? "s" : " "));
                System.out.print((breeze[x][y] ? "b" : " "));
            }
            System.out.println("|");
        }
    }

    // Variable to hold user input
    private static int ch = 0;
    private static BufferedReader bufferedReader;

    /**
     * Routine to read next meaningful character
     * @return
     * @throws IOException
     */
    private static int getNextChar() throws IOException {
        if (bufferedReader == null) {
            //  open up standard input
            bufferedReader = new BufferedReader(new InputStreamReader(System.in));
        }

        do {
            try {
                ch = bufferedReader.read();
            } catch (EOFException e) {
                return -1;
            } catch (IOException e) {
                throw e;
            }
        } while (Character.isWhitespace(ch));

        bufferedReader.readLine();

        return ch;
    }

    /**
     * Routine to run a game using user input
     *
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if (args.length == 0)
            initialize(3);
        else if (args.length == 1) {
            try {
                initialize((int) Integer.parseInt(args[0]));
            }
            catch (Exception e) {
                System.out.println("Problem parsing matrix size " + args[0]);
                return;
            }
        }
        else {
            System.out.println("Cannot use more than one argument");
            return;
        }

        HashSet legalChars = new HashSet();
        legalChars.add((int) 'f'); legalChars.add((int) 'F');
        legalChars.add((int) 'l'); legalChars.add((int) 'L');
        legalChars.add((int) 'r'); legalChars.add((int) 'R');
        legalChars.add((int) 'g'); legalChars.add((int) 'G');
        legalChars.add((int) 's'); legalChars.add((int) 'S');

        while (!maze.gameOver()) {
            currentX = maze.currentX();
            currentY = maze.currentY();
            facingAngle = maze.facingAngle();
            hasArrow = maze.hasArrow();
            visited[currentX][currentY] = true;
            glitter[currentX][currentY] = maze.hasGlitter();
            smell[currentX][currentY] = maze.hasSmell();
            breeze[currentX][currentY] = maze.hasBreeze();
            showBoard();
            // Replace code starting here
            System.out.print("F)orward, L)eft, R)ight, G)rab, S)hoot: ");
            do {
                try {
                    ch = getNextChar();
                }
                catch (IOException e) {
                    System.out.println("Do not understand character");
                }
            } while (legalChars.contains(ch) == false);
            // By here you should have the system select working code
            switch (ch) {
                case 'f': case 'F': maze.doAction(WumpusMaze.FORWARD_ACTION); break;
                case 'l': case 'L': maze.doAction(WumpusMaze.LEFT_ACTION); break;
                case 'r': case 'R': maze.doAction(WumpusMaze.RIGHT_ACTION); break;
                case 'g': case 'G': maze.doAction(WumpusMaze.GRAB_ACTION); break;
                case 's': case 'S': maze.doAction(WumpusMaze.SHOOT_ACTION); break;
            }
        }
        System.out.println("Total reward is " + maze.totalReward());
    }

}
